import asyncio
import streamlit as st
import pyvista as pv

from stpyvista import experimental_vtkjs
from stpyvista.export import export_vtksz

@st.cache_resource
def create_plotter(dummy:str = "sphere"):
    
    # Initialize a plotter object
    plotter = pv.Plotter(shape=(2,1))
    mesh = pv.Sphere(radius=1.0, center=(0, 0, 0))
    
    x, y, z = mesh.cell_centers().points.T
    mesh["My scalar"] = z

    ## Add mesh to the plotter
    plotter.add_mesh(
        mesh,
        scalars="My scalar",
        cmap="prism",
        show_edges=True,
        edge_color="#001100",
        ambient=0.2,
        show_scalar_bar = False
    )
        
    ## Some final touches
    plotter.view_isometric()
    plotter.background_color = "pink"
    
    return plotter

async def main():

    st.set_page_config(page_icon="🧊", layout="wide")
    st.title("🧊 `stpyvista`")
    st.write("*Show PyVista 3D visualizations in Streamlit*")
    
    plotter = create_plotter()

    if "data" not in st.session_state:
        st.session_state.data = await export_vtksz(plotter)
    
    lcol, rcol = st.columns(2)
    with rcol:
        "🌎 3D Model"
        camera = experimental_vtkjs(st.session_state.data, key="experimental-stpv")
        
    with lcol:
        "🎥 Camera"
        st.json(camera)

if __name__ == "__main__":
    asyncio.run(main())


import urllib.parse as parse
from subprocess import run
from functools import partial
from pyvista import start_xvfb as pv_start_xvfb
from streamlit.runtime.scriptrunner import get_script_run_ctx
from datetime import datetime

_run_command = partial(run, capture_output=True, text=True)


def is_the_app_embedded():
    """Check if the app is embedded based on the query parameters"""

    ctx = get_script_run_ctx()
    query_params = parse.parse_qs(ctx.query_string)
    return True if query_params.get("embed") else False


def _is_pgrep_installed():
    """
    Check if `pgrep` is installed in the environment
    """
    is_pgrep_installed = _run_command(["which", "pgrep"])
    return True if is_pgrep_installed.returncode == 0 else False


def start_xvfb():
    """
    This is a thin wrapper over pv.start_xvfb().
    Check if virtual framebuffer Xvfb is already running on the machine and starts it if not.
    Only available on Linux. Be sure to install `libgl1-mesa-glx` and `xvfb` in your package manager.
    """

    if not _is_pgrep_installed():
        raise OSError(
            "pgrep is not installed. Be sure to install `procps` in your package manager."
        )

    is_xvfb_running = _run_command(["pgrep", "Xvfb"])
    print(datetime.now().strftime(r"%y-%m-%d %H:%M:%S"))
    
    if is_xvfb_running.returncode == 1:
        print("--> Initialize")
        pv_start_xvfb()

    elif is_xvfb_running.returncode == 0:
        print(f"--> PID: {is_xvfb_running.stdout.strip()}")

    else:
        raise OSError(
            "Something went wrong checking for the machine processes"
            f"{is_xvfb_running.stdout}"
            f"{is_xvfb_running.stderr}"
        )




from trame.app import get_server
from trame.widgets import (
    vuetify as vuetify,
)

from trame.widgets.vtk import VtkLocalView
from trame.ui.vuetify import SinglePageLayout

from pyvista.plotting import Plotter

SERVER_NAME = "stpyvista_server"


async def export_vtksz(plotter: Plotter):
    """Export this plotter as a VTK.js OfflineLocalView file.

    Parameters
    ----------
    plotter : Plotter
        PyVista Plotter object.

    Returns
    -------
    bytes
        The exported plotter view.
    """

    # Get a trame server and launch it
    server = get_server(name=SERVER_NAME, client_type="vue2")
    _, ctrl = server.state, server.controller

    with SinglePageLayout(server) as layout:
        with layout.content:
            view = VtkLocalView(plotter.ren_win)
            ctrl.view_update = view.update

    server.start(
        exec_mode="task",
        host="127.0.0.1",
        port="0",
        open_browser=False,
        show_connection_info=False,
        disable_logging=True,
        timeout=0,
        backend="tornado",
    )

    content = view.export(format=format)
    view.release_resources()

    return content





import streamlit as st
import pyvista as pv
import numpy as np
from stpyvista import stpyvista, dataview

def put_in_plotter(actor: pv.DataSet):
    plotter = pv.Plotter()
    plotter.window_size = (300, 300)
    plotter.add_mesh(actor, line_width=10)
    plotter.view_isometric()
    return plotter

def sphere():
    return pv.Sphere(radius=1.0, center=(0, 0, 0))

def spline():
    theta = np.linspace(-1 * np.pi, 1 * np.pi, 100)
    z = np.linspace(2, -2, 100)
    r = z**2 + 1
    x = r * np.sin(theta)
    y = r * np.cos(theta)
    points = np.column_stack((x, y, z))
    return pv.Spline(points, 1000)

def surface():
    x = np.arange(-10, 10, 0.5)
    y = np.arange(-10, 10, 0.5)
    x, y = np.meshgrid(x, y)
    z = np.sin(np.sqrt(x**2 + y**2))
    
    surf = pv.StructuredGrid(x, y, z)
    
    x, y, z = surf.cell_centers().points.T
    surf["x"] = x
    surf["y"] = y
    surf["z"] = z
    surf["r"] = np.sqrt(x**2 + y**2 + z**2)

    surf.set_active_scalars("r")

    return surf


def main():

    st.title("Testing `dataview`")

    datasets = [
        sphere(),
        spline(),
        surface()
    ]
    
    for obj in datasets:
        cols = st.columns([1, 1.5])
        
        with cols[0]:
            stpyvista(put_in_plotter(obj))
        with cols[1]:
            dataview(obj)
    
    dataview(pv.MultiBlock(datasets))

if __name__ == "__main__":
    main()


# __init__.py

from io import StringIO
from pathlib import Path
from typing import Optional, Literal
import base64
import xml.dom.minidom

import streamlit as st
import streamlit.components.v1 as components

from pyvista.plotting import Plotter
from pyvista import DataSet

import panel as pn
from bokeh.resources import CDN, INLINE

pn.extension("vtk", sizing_mode="stretch_both")
BOKEH_RESOURCES = {"CDN": CDN, "INLINE": INLINE}


class stpyvistaTypeError(TypeError):
    pass


class stpyvistaValueError(ValueError):
    pass


# Tell streamlit that there is a component called `experimental_vtkjs`,
# and that the code to display that component is in the "vanilla_vtkjs" folder
experimental_frontend_dir = (Path(__file__).parent / "vanilla_vtkjs").absolute()
_exp_component_func = components.declare_component(
    "experimental_vtkjs", path=str(experimental_frontend_dir)
)


def experimental_vtkjs(
    vtksz_data: bytes, height: int = 400, key: Optional[str] = None
) -> dict:
    """
    Renders an interactive Pyvista Plotter in streamlit.

    Parameters
    ----------
    vtksz_data: bytes
        Data from a vtksz in zip format.

    Returns
    -------
    dict
        A dictionary with the current Camera view properties.
    """

    base64_str = base64.b64encode(vtksz_data).decode().replace("\n", "")

    component_value = _exp_component_func(
        plotter_data=base64_str,
        height=str(height),
        key=key,
        default=0,
    )

    return component_value


frontend_dir = (Path(__file__).parent / "panel_based").absolute()
_component_func = components.declare_component("stpyvista", path=str(frontend_dir))


def stpyvista(
    plotter: Plotter,
    use_container_width: bool = True,
    horizontal_align: Literal["center", "left", "right"] = "center",
    panel_kwargs: Optional[dict] = None,
    bokeh_resources: Literal["CDN", "INLINE"] = "INLINE",
    key: Optional[str] = None,
) -> None:
    """
    Renders an interactive Pyvista Plotter in streamlit.
    
    Parameters
    ----------
    input: pv.Plotter
        Pyvista plotter object to render.
    
    use_container_width : bool = True
        If True, set the dataframe width to the width of the parent container. \
        This takes precedence over the `horizontal_align` argument. \
        Defaults to `True`.
    
    horizontal_align : Literal["center", "left", "right"] = "center"
        Horizontal alignment of the stpyvista component. This parameter is ignored if 
        `use_container_width = True`. Defaluts to `"center"`.

    panel_kwargs : dict | None = None
        Optional keyword parameters to pass to pn.panel() Check: 
        https://panel.holoviz.org/api/panel.pane.vtk.html for details. Here is
        a useful one:
        
        orientation_widget : bool
            Show the xyz axis indicator

    bokeh_resources: Literal["CDN", "INLINE"] = "Inline"
        Source of the BokehJS configuration. Check:
        https://docs.bokeh.org/en/latest/docs/reference/resources.html for details. \
        Defaults to "INLINE" 

    key: str|None
        An optional key that uniquely identifies this component. If this is
        None, and the component's arguments are changed, the component will
        be re-mounted in the Streamlit frontend and lose its current state.
    
    Returns
    -------
    None

    """

    if isinstance(plotter, Plotter):
        if panel_kwargs is None:
            panel_kwargs = dict()

        width, height = plotter.window_size

        geo_pan_pv = pn.panel(plotter.ren_win, **panel_kwargs)

        # Check bokeh_resources
        if bokeh_resources not in ("CDN", "INLINE"):
            raise stpyvistaValueError(
                f'"{bokeh_resources}" is not a valid bokeh resource. '
                'Valid options are "CDN" or "INLINE".'
            )

        # Create HTML file
        with StringIO() as model_bytes:
            geo_pan_pv.save(model_bytes, resources=BOKEH_RESOURCES[bokeh_resources])
            panel_html = model_bytes.getvalue()

        component_value = _component_func(
            panel_html=panel_html,
            height=height,
            width=width,
            horizontal_align=horizontal_align,
            use_container_width=1 if use_container_width else 0,
            bgcolor=plotter.background_color.hex_rgba,
            key=key,
            default=0,
        )

        return component_value

    else:
        raise stpyvistaTypeError(f"{plotter} is not a `pyvista.Plotter` instance. ")


def dataview(obj: DataSet):
    """
    Renders the HTML representation of a Pyvista/VTK dataset.

    Parameters
    ----------
    element: pv.DataSet
        Pyvista element with some data to show.

    Returns
    -------
    None
    """

    def assemble_details(title: str, table: str) -> str:
        return f"<details open><summary><em>{title}</em></summary>{table}</details>"

    try:
        # Look up an HTML representation and arange in details tags

        dom = xml.dom.minidom.parseString(obj._repr_html_())
        tables = dom.getElementsByTagName("table")

        css = """
            <style>
            details {
                padding: 6px;
                margin-bottom: 5px; 
                border: 1px solid #eeeeee;
                background-color: transparent;
                border-radius: 10px;
            }
            summary {
                background-color: transparent;
                opacity: 60%;
                padding: 5px;
            }
            summary::marker {
                color: purple;
                font-size: 1.1em;
            }
            </style>
        """

        html = StringIO("""<div class="stpv-dataview">""")

        if len(tables) == 1:
            html.write(assemble_details("Header", tables[0].toprettyxml()))

        else:
            headers = (
                dom.getElementsByTagName("table")[0]
                .getElementsByTagName("tr")[0]
                .getElementsByTagName("th")
            )

            for title, table in zip(headers, tables[1:]):
                html.write(
                    assemble_details(title.firstChild.nodeValue, table.toprettyxml())
                )

        html.write(css + "</div>")

        # Render in streamlit
        st.html(html.getvalue())

    except AttributeError:
        # Defaults to streamlit's write function
        st.write(obj)


def main():
    pass


if __name__ == "__main__":
    main()



import streamlit as st
import pyvista as pv
from stpyvista import stpyvista

st.set_page_config(page_icon="🧊", layout="wide")
st.title("🧊 `stpyvista`")
st.sidebar.header("Show PyVista 3D visualizations in Streamlit")

## Initialize a plotter object
plotter = pv.Plotter(window_size=[500,400])

## Create a mesh with a cube 
mesh = pv.Cube(center=(0,0,0))

## Add some scalar field associated to the mesh
mesh['myscalar'] = mesh.points[:, 2]*mesh.points[:, 0]
mesh['otherscalar'] = mesh.points[:, 2]**2 + mesh.points[:, 0]
mesh.set_active_scalars('otherscalar')

## Add mesh to the plotter
plotter.add_mesh(mesh, cmap='bwr', line_width=1,  label="cube")
plotter.add_scalar_bar()
plotter.add_title("My cube", color="black", font_size=14)

## Final touches
plotter.view_isometric()
plotter.background_color = 'lightgray'

chk = st.sidebar.checkbox("use_container_width", False)
align = st.sidebar.select_slider("align", ["left", "center", "right"])

## Pass a key to avoid re-rendering at each time something changes in the page
stpyvista(
    plotter, 
    use_container_width=chk, 
    panel_kwargs=dict(orientation_widget = True),
    horizontal_align=align
)

## Add something else below
st.markdown("Jello there"*50)

cols = st.columns([1,2,1], gap="small")

for col in cols:
    with col:
        stpyvista(plotter, use_container_width=chk, bokeh_resources="CDN")